@@ -1,10 +1,11 @@
 package Strehler::Admin;
-$Strehler::Admin::VERSION = '1.5.5';
+$Strehler::Admin::VERSION = '1.6.0';
 use strict;
 use Cwd 'abs_path';
 use Dancer2 0.154000;
 use Dancer2::Plugin::DBIC;
 use Dancer2::Plugin::Ajax;
+use Dancer2::Serializer::JSON;
 use Strehler::Dancer2::Plugin::Admin;
 use HTML::FormFu 1.00;
 use HTML::FormFu::Element::Block;
@@ -34,6 +35,8 @@ my $module_file_path = __FILE__;
 my $root_path = abs_path($module_file_path);
 $root_path =~ s/Admin\.pm//;
 
+my $forms_path = $root_path . 'forms';
+
 set views => $root_path . 'views';
 
 ##### Homepage #####
@@ -112,14 +115,15 @@ any '/user/add' => sub
     if($form->submitted_and_valid)
     {
         my $id = Strehler::Element::User->save_form(undef, $form);
-        if($id == -1)
-        {
-            $message = "Username already in use";
-        }
-        else
+        if($id >= 0)
         {
             Strehler::Element::Log->write(session->read('user'), 'add', 'user', $id);
             redirect dancer_app->prefix . '/user/list';
+        
+        }
+        else
+        {
+            return template "admin/message", { message => Strehler::Element::User->error_message("add", $id), backlink => dancer_app->prefix . '/user' }; 
         }
     }
     template "admin/user", { form => $form->render(), message => $message }
@@ -146,15 +150,15 @@ post '/user/edit/:id' => sub
     if($form->submitted_and_valid)
     {
         my $return_id = Strehler::Element::User->save_form($id, $form);
-        if($return_id == -1)
-        {
-            $message = "Username already in use";
-        }
-        else
+        if($id >= 0)
         {
             Strehler::Element::Log->write(session->read('user'), 'edit', 'user', $id);
             redirect dancer_app->prefix . '/user/list';
         }
+        else
+        {
+            return template "admin/message", { message => Strehler::Element::User->error_message("edit", $id), backlink => dancer_app->prefix . '/user' }; 
+        }
     }
     template "admin/user", { form => $form->render(), message => $message }
 };
@@ -178,9 +182,9 @@ post '/user/password' => sub
     if($form->submitted_and_valid)
     {
         my $return_id = Strehler::Element::User->save_password($id, $form);
-        if($return_id == -1)
+        if($return_id < 0)
         {
-            $message = "Username already in use";
+            return template "admin/message", { message => Strehler::Element::User->error_message("change password", $id), backlink => dancer_app->prefix . '/user' }; 
         }
         else
         {
@@ -203,10 +207,10 @@ get '/category' => sub
 
 any '/category/list' => sub
 {
-    send_error("Access denied", 403) && return && return if ( ! Strehler::Meta::Category->check_role(session->read('role')));
+    send_error("Access denied", 403) && return if ( ! Strehler::Meta::Category->check_role(session->read('role')));
 
     #THE TABLE
-    my $to_view = Strehler::Meta::Category->get_list();
+    my @to_view = Strehler::Meta::Category->get_list();
     my @entities = Strehler::Helpers::get_categorized_entities();
 
     #THE FORM
@@ -217,9 +221,9 @@ any '/category/list' => sub
     {
         my $id = Strehler::Meta::Category->save_form(undef, $form, \@entities);
         Strehler::Element::Log->write(session->read('user'), 'add', 'category', $id);
-        redirect dancer_app->prefix . '/category/list';
+        redirect dancer_app->prefix . '/category/list?message=fast-created';
     }
-    template "admin/category_list", { categories => $to_view, form => $form };
+    template "admin/category_list", { categories => \@to_view, form => $form, "message" => params->{'message'}};
 };
 
 any '/category/add' => sub
@@ -271,33 +275,47 @@ get '/category/delete/:id' => sub
     send_error("Access denied", 403) && return if ( ! Strehler::Meta::Category->check_role(session->read('role')));
     my $id = params->{id};
     my $category = Strehler::Meta::Category->new($id);
-    if($category->has_elements())
-    {
-        my $message = "Category " . $category->get_attr('category') . " is not empty! Deletion is impossible.";    
-        my $return = dancer_app->prefix . "/category/list";
-        template "admin/message", { message => $message, backlink => $return };
-    }
-    elsif($category->is_parent())
-    {
-        my $message = "Category " . $category->get_attr('category') . " has subcategories! Deletion is impossible.";    
-        my $return = dancer_app->prefix . "/category/list";
-        template "admin/message", { message => $message, backlink => $return };
-    }
-    else
-    {
-        my %data = $category->get_basic_data();
-        template "admin/delete", { what => "category", el => \%data, backlink => dancer_app->prefix . '/category' };
-    }
+    my %data = $category->get_basic_data();
+    template "admin/delete-category", { what => "category", subs => $category->is_parent(), el => \%data, backlink => dancer_app->prefix . '/category' };
 };
 post '/category/delete/:id' => sub
 {
     send_error("Access denied", 403) && return if ( ! Strehler::Meta::Category->check_role(session->read('role')));
     my $id = params->{id};
     my $category = Strehler::Meta::Category->new($id);
-    $category->delete();
+    my $code = $category->delete();
+    if($code != 0)
+    {
+        return template "admin/message", { message => $category->error_message("delete", $code), backlink => dancer_app->prefix . '/category' }; 
+    }
     Strehler::Element::Log->write(session->read('user'), 'delete', 'category', $id);
     redirect dancer_app->prefix . '/category/list';
 };
+post '/category/delete-tree/:id' => sub
+{
+    send_error("Access denied", 403) && return if ( ! Strehler::Meta::Category->check_role(session->read('role')));
+    my $id = params->{id};
+    my $category = Strehler::Meta::Category->new($id);
+    my @categories_to_delete = ( $category );
+    while(@categories_to_delete)
+    {
+        my $cat = shift @categories_to_delete;
+        if($cat->is_parent())
+        {
+            @categories_to_delete = ( $cat->subcategories(0), $cat, @categories_to_delete );
+        }
+        else
+        {
+            my $code = $cat->delete();
+            if($code != 0)
+            {
+                return template "admin/message", { message => $cat->error_message("delete", $code), backlink => dancer_app->prefix . '/category' }; 
+            }
+            Strehler::Element::Log->write(session->read('user'), 'delete', 'category', $cat->get_attr('id'));
+        }
+    }
+    redirect dancer_app->prefix . '/category/list';
+};
 
 ajax '/category/select/:id' => sub
 {
@@ -327,6 +345,57 @@ ajax '/category/select' => sub
         return 0;
     }
 };
+ajax '/category/info' => sub
+{
+    content_type('application/json');
+    my $category;
+    my $input = params->{input};
+    my $option = params->{option} || undef;
+    my $ancestor = 0;
+    if(params->{query} eq 'id' && $input)
+    {
+        if($input =~ m/^anc:([0-9]+)$/)
+        {
+            $input = $1;
+            $ancestor = 1;
+        }
+        $category = Strehler::Meta::Category->new($input);
+    }
+    elsif(params->{query} eq 'name' && $input)
+    {
+        if($input =~ m/^(.*)\*$/)
+        {
+            $input = $1;
+            $ancestor = 1;
+        }
+        $category = Strehler::Meta::Category->explode_name($input);
+    }
+    else
+    {
+        $category = Strehler::Meta::Category->new( row => undef );
+    }
+    my %data = $category->get_basic_data();
+    my $subs = Strehler::Meta::Category->make_select($category->get_attr('id'), $option);
+    my @subs_array = @{$subs};
+    $data{select} = template 'admin/category_select', { categories => $subs }, { layout => undef };
+    if($#subs_array > 0)
+    {
+        $data{subcategories} = 1;
+    }
+    else
+    {
+        $data{subcategories} = 0;
+    }
+    if($ancestor && $category->exists())
+    {
+        $data{'parent'} = $input;
+        $data{subcategories} = 0;
+        $data{'ext_name'} = $data{'ext_name'} . "/*";
+        $data{'id'} = 'anc:'.$input;
+    }
+    my $serializer = Dancer2::Serializer::JSON->new();
+    return $serializer->serialize(\%data);
+};
 ajax '/category/tagform/:type/:id?' => sub
 {
     content_type('text/plain');
@@ -373,6 +442,7 @@ get '/:entity' => sub
 
 any '/:entity/list' => sub
 {
+    #Page init
     my $entity = params->{entity};
     my $class = Strehler::Helpers::class_from_entity($entity);
     if(! $class->auto())
@@ -381,80 +451,64 @@ any '/:entity/list' => sub
     }
     send_error("Access denied", 403) && return if ( ! $class->check_role(session->read('role')));
 
+    #Parameters collection
+    my $input_params;
+    my $session;
+    foreach my $p (Strehler::Helpers::list_parameters_names('session'))
+    {
+        if(exists params->{$p})
+        {
+            $input_params->{$p} = params->{$p};
+        }
+        my $session_value = session->read($entity . "-" . $p);
+        $session->{$p} = $session_value if($session_value);
+    }
+    foreach my $p (Strehler::Helpers::list_parameters_names('extra'))
+    {
+        if(exists params->{$p})
+        {
+            $input_params->{$p} = params->{$p};
+        }
+    }
+    my %parameters = Strehler::Helpers::list_parameters_init($entity, $session, $input_params);
+    if(exists $parameters{'error'} && $parameters{'error'} == 1)
+    {
+       my $backlink = params->{'strehl-from'} || "/admin/$entity/list";
+       return template "admin/message", { message => "Wrong category name: " . params->{'strehl-catname'}, backlink => $backlink }; 
+    }
+
+    #Search
+    my $elements;
+    my %search_params = %parameters;
+    if($parameters{'search'})
+    {
+        $elements = $class->search_box($parameters{'search'}, \%search_params);
+    }
+    else
+    {
+        $elements = $class->get_list(\%search_params);
+    }
+    my $filter_form = Strehler::Forms::form_filter($forms_path . '/admin/category_filter.yml', $class->multilang(), $parameters{'language'}, \@languages);
+
+    #Session saving management
+    foreach my $p_save (Strehler::Helpers::list_parameters_names('session'))
+    {
+        session $entity . '-' . $p_save => $parameters{$p_save};
+    }
+
+    #Rendering
     my $custom_list_template = $class->custom_list_template();
     my $list_view = $custom_list_template ? 'admin/custom_list' : 'admin/generic_list';
-    
-    my $page = exists params->{'page'} ? params->{'page'} : session $entity . '-page';
-    my $cat_param = exists params->{'cat'} ? params->{'cat'} : session $entity . '-cat-filter';
-    my $order = exists params->{'order'} ? params->{'order'} : session $entity . '-order';
-    my $order_by = exists params->{'order-by'} ? params->{'order-by'} : session $entity . '-order-by';
-    my $search = exists params->{'search'} ? params->{'search'} : session $entity . '-search';
-    my $ancestor = exists params->{'ancestor'} ? params->{'ancestor'} : session $entity . '-ancestor';
-    my $language = exists params->{'language'} ? params->{'language'} : session $entity . '-language';
-    my $wanted_cat = undef;
-    if(exists params->{'strehl-catname'})
-    {
-        $wanted_cat = Strehler::Meta::Category->explode_name(params->{'strehl-catname'});
-        if(! $wanted_cat->exists())
-        {
-           my $backlink = params->{'strehl-from'} || "/admin/$entity/list";
-           return template "admin/message", { message => "No elements in category: " . params->{'strehl-catname'}, backlink => $backlink }; 
-        }
-        $cat_param = $wanted_cat->get_attr('id');
-    }
-    else
-    {
-        if($cat_param)
-        {
-            $wanted_cat = Strehler::Meta::Category->new($cat_param);
-        }
-    }
-    my $backlink = params->{'strehl-from'};
-    my $cat = undef;
-    my $subcat = undef;
-    if($wanted_cat)
-    {
-        if($wanted_cat->row->parent)
-        {
-            $cat = $wanted_cat->row->parent->id;
-            $subcat = $wanted_cat->row->id;
-        }
-        else
-        {
-            $cat = $wanted_cat->row->id;
-        }
-    }
-    if(exists params->{'cat_name'} || exists params->{'cat'})
-    {
-        $ancestor = undef;
-    }
-    elsif(exists params->{'ancestor'})
-    {
-        $cat_param = undef;
-        $cat = $ancestor;
-        $subcat = '*';
-    }
-    $page ||= 1;
-    $order ||= 'desc';
-    my $entries_per_page = 20;
-    my $search_parameters = { page => $page, entries_per_page => $entries_per_page, category_id => $cat_param, ancestor => $ancestor, order => $order, order_by => $order_by, language => $language};
-    my $elements;
-    if($search)
-    {
-        $elements = $class->search_box($search, $search_parameters);
-    }
-    else
-    {
-        $elements = $class->get_list($search_parameters);
-    }
-    session $entity . '-page' => $page;
-    session $entity . '-cat-filter' => $cat_param;
-    session $entity . '-order' => $order;
-    session $entity . '-order-by' => $order_by;
-    session $entity . '-search' => $search;
-    session $entity . '-ancestor' => $ancestor;
-    session $entity . '-language' => $language;
-    template $list_view, { entity => $entity, elements => $elements->{'to_view'}, page => $page, cat_filter => $cat, subcat_filter => $subcat, search => $search, order => $order, order_by => $order_by, fields => $class->fields_list(), last_page => $elements->{'last_page'}, $class->entity_data(), custom_list_template => $custom_list_template, backlink => $backlink, language => $language, languages => \@languages };
+    template $list_view, { entity => $entity, 
+                           elements => $elements->{'to_view'}, 
+                           last_page => $elements->{'last_page'}, 
+                           filter_form => $filter_form, 
+                           fields => $class->fields_list(), 
+                           custom_list_template => $custom_list_template, 
+                           languages => \@languages, 
+                           %parameters,
+                           $class->entity_data()
+                          }; 
 };
 get '/:entity/turnon/:id' => sub
 {
@@ -468,7 +522,11 @@ get '/:entity/turnon/:id' => sub
     send_error("Access denied", 403) && return if ( ! $class->check_role(session->read('role')));
     my $id = params->{id};
     my $obj = $class->new($id);
-    $obj->publish();
+    my $code = $obj->publish();
+    if($code != 0)
+    {
+       return template "admin/message", { message => $obj->error_message("publish", $code), backlink => dancer_app->prefix . '/' . $entity }; 
+    }
     Strehler::Element::Log->write(session->read('user'), 'publish', $entity, $id);
     redirect $redirect;
 };
@@ -484,7 +542,11 @@ get '/:entity/turnoff/:id' => sub
     send_error("Access denied", 403) && return if ( ! $class->check_role(session->read('role')));
     my $id = params->{id};
     my $obj = $class->new($id);
-    $obj->unpublish();
+    my $code = $obj->unpublish();
+    if($code != 0)
+    {
+       return template "admin/message", { message => $obj->error_message("unpublish", $code), backlink => dancer_app->prefix . '/' . $entity }; 
+    }
     Strehler::Element::Log->write(session->read('user'), 'unpublish', $entity, $id);
     redirect $redirect;
 };
@@ -513,7 +575,11 @@ post '/:entity/delete/:id' => sub
     send_error("Access denied", 403) && return if ( ! $class->check_role(session->read('role')));
     my $id = params->{id};
     my $obj = $class->new($id);
-    $obj->delete();
+    my $code = $obj->delete();
+    if($code != 0)
+    {
+       return template "admin/message", { message => $obj->error_message("delete", $code), backlink => dancer_app->prefix . '/' . $entity }; 
+    }
     Strehler::Element::Log->write(session->read('user'), 'delete', $entity, $id);
     redirect dancer_app->prefix . '/' . $entity . '/list';
 };
@@ -599,6 +665,10 @@ any '/:entity/add' => sub
     if($form->submitted_and_valid)
     {
         my $id = $class->save_form(undef, $form, request->uploads());
+        if($id < 0)
+        {
+            return template "admin/message", { message => $class->error_message("delete", $id), backlink => dancer_app->prefix . '/' . $entity }; 
+        }
         Strehler::Element::Log->write(session->read('user'), 'add', $entity, $id);
         my $action = params->{'strehl-action'};
         if(! $action)
@@ -639,15 +709,7 @@ any '/:entity/add' => sub
             $wanted_cat = Strehler::Meta::Category->explode_name(params->{'strehl-catname'});
             if($wanted_cat->exists())
             {
-                my $parent = $wanted_cat->get_attr('parent');
-                if($parent)
-                {
-                    $form->default_values({ category => $parent, subcategory => $wanted_cat->get_attr('id')});
-                }
-                else
-                {
-                    $form->default_values({ category => $wanted_cat->get_attr('id')});
-                }
+                $form->default_values({ category => $wanted_cat->get_attr('id')});
             }    
         }
         if(exists params->{'strehl-today'})
@@ -738,7 +800,12 @@ post '/:entity/edit/:id' => sub
     my $message = 'quiet';
     if($form->submitted_and_valid)
     {
-        $class->save_form($id, $form, request->uploads());
+        my $id = $class->save_form($id, $form, request->uploads());
+        if($id < 0)
+        {
+            return template "admin/message", { message => $class->error_message("edit", $id), backlink => dancer_app->prefix . '/' . $entity }; 
+        }
+
         Strehler::Element::Log->write(session->read('user'), 'edit', $entity, $id);
         my $action = params->{'strehl-action'};
         if(! $action)
