@@ -1,5 +1,5 @@
 package Strehler::Element;
-$Strehler::Element::VERSION = '1.5.5';
+$Strehler::Element::VERSION = '1.6.0';
 use strict;
 use Carp 'carp';
 use Moo;
@@ -66,6 +66,7 @@ sub get_attr
     my $self = shift;
     my $attribute = shift;
     my $bare = shift || 0;
+    return undef if(! $self->exists);
     my $accessor = $self->can($attribute);
     if($accessor && ! $bare)
     {
@@ -80,7 +81,14 @@ sub get_attr
     }
     if($attribute eq 'category')
     {
-        return $self->row->category->category;
+        if($self->row->category)
+        {
+            return $self->row->category->category;
+        }
+        else
+        {
+            return undef;
+        }
     }
     if($attribute eq 'main-title')
     {
@@ -204,10 +212,17 @@ sub get_tags
 sub get_category_name
 {
     my $self = shift;
-    if($self->row->can('category'))
+    if($self->row && $self->row->can('category'))
     {
-            my $category = Strehler::Meta::Category->new($self->row->category->id);
-            return $category->ext_name();
+            if($self->row->category)
+            {
+                my $category = Strehler::Meta::Category->new($self->row->category->id);
+                return $category->ext_name();
+            }
+            else
+            {
+                return undef;
+            }
     }
     else
     {
@@ -403,6 +418,7 @@ sub publish
     return if ! $self->publishable();
     $self->row->published(1);
     $self->row->update();
+    return 0;
 }
 sub unpublish
 {
@@ -410,6 +426,7 @@ sub unpublish
     return if ! $self->publishable();
     $self->row->published(0);
     $self->row->update();
+    return 0;
 }
 sub next_in_category_by_order
 {
@@ -712,12 +729,12 @@ sub get_list
     my @languages_list = undef;
     if($args{'language'})
     {
+        #Languages arrive here as an array of strings
         $forced_language = 1;
         if($self->multilang_children())
         {
             push @{$args{'join'}}, $self->multilang_children();
         }
-        @languages_list = split(',',  $args{'language'});
     }
     else
     {
@@ -730,8 +747,7 @@ sub get_list
     {
         if($self->multilang_children())
         {
-            my $languages_query;
-            $search_criteria->{$self->multilang_children() . '.language'} = \@languages_list;
+            $search_criteria->{$self->multilang_children() . '.language'} = { -in => $args{'language'} };
         }
     }
     if($args{'order_by'} =~ /^(.*?)\.(.*?)$/)
@@ -801,7 +817,7 @@ sub get_list
         my $category_obj = Strehler::Meta::Category->new($args{'ancestor'});
         my @category_ids;
         push @category_ids, $args{'ancestor'};
-        my @subcategories = $category_obj->subcategories;
+        my @subcategories = $category_obj->subcategories(1);
         for(@subcategories)
         {
             push @category_ids, $_->get_attr('id');
@@ -891,14 +907,13 @@ sub get_form_data
     }
     if($self->categorized()) #Is the element categorized?
     {
-        if($el_row->category->parent)
+        if($el_row->category)
         {
-            $data->{'category'} = $el_row->category->parent->id;
-            $data->{'subcategory'} = $el_row->category->id;
+            $data->{'category'} = $el_row->category->id;
         }
         else
         {
-            $data->{'category'} = $el_row->category->id;
+            $data->{'category'} = undef;
         }
     }
     $data->{'tags'} = $self->get_tags();
@@ -992,16 +1007,10 @@ sub save_form
         }
         elsif($column eq 'category')
         {
-            my $category;
-            if($form->param_value('subcategory'))
+            if($form->param_value('category'))
             {
-                $category = $form->param_value('subcategory');
+                $el_data->{'category'} = $form->param_value('category');
             }
-            elsif($form->param_value('category'))
-            {
-                $category = $form->param_value('category');
-            }
-            $el_data->{'category'} = $category;
         }
     }
     if($self->publishable() && $self->auto_publish() && ! $id)
@@ -1115,6 +1124,21 @@ sub check_role
     }
 }
 
+sub error_message
+{
+    my $self = shift;
+    my $action = shift;
+    my $code = shift;
+    if($code == '0')
+    {
+        return "OK";
+    }
+    else
+    {
+        return "An error has occurred";
+    }
+}
+
 
 
 1;
