@@ -1,5 +1,5 @@
 package Strehler::Element::Log;
-$Strehler::Element::Log::VERSION = '1.5.5';
+$Strehler::Element::Log::VERSION = '1.6.0';
 use strict;
 use Moo;
 use Dancer2 0.154000;
@@ -102,7 +102,7 @@ sub fields_list
 }
 sub custom_list_template
 {
-    return $views_path . "/admin/blocks/log_list_block.tt";
+    return $views_path . "/admin/entities/log_list_block.tt";
 }
 
 sub install
