@@ -1,5 +1,5 @@
 package Strehler::FormFu::Validator::CategoryUnique;
-$Strehler::FormFu::Validator::CategoryUnique::VERSION = '1.5.5';
+$Strehler::FormFu::Validator::CategoryUnique::VERSION = '1.6.0';
 use strict;
 use warnings;
 use Moose;
@@ -22,7 +22,7 @@ sub validate_value {
     if($parent)
     {
         my $parent_element = Strehler::Meta::Category->new($parent);
-        $category_name = $parent_element->get_attr('category') . '/' . $category;
+        $category_name = $parent_element->ext_name() . '/' . $category;
     }
     else
     {
