@@ -1,5 +1,5 @@
 package Strehler::Forms;
-$Strehler::Forms::VERSION = '1.5.5';
+$Strehler::Forms::VERSION = '1.6.0';
 use Cwd 'abs_path';
 use HTML::FormFu;
 
@@ -24,8 +24,6 @@ sub form_category
     my $form = HTML::FormFu->new;
     $form->auto_error_class('error-msg');
     $form->load_config_file( $form_path . '/admin/category.yml' );
-    my $category = $form->get_element({ name => 'parent'});
-    $category->options(Strehler::Meta::Category->make_select());
     $form = add_dynamic_fields_for_category($form); 
     my $prev_name = $form->element( { type => 'Hidden', name => 'prev-name'} );
     my $prev_parent = $form->element( { type => 'Hidden', name => 'prev-parent'} );
@@ -40,8 +38,6 @@ sub form_category_fast
     my $form = HTML::FormFu->new;
     $form->auto_error_class('error-msg');
     $form->load_config_file( $form_path . '/admin/category_fast.yml' );
-    my $parent = $form->get_element({ name => 'parent'});
-    $parent->options(Strehler::Meta::Category->make_select());
     return $form;
 }
 
@@ -59,7 +55,9 @@ sub form_user
     }
     elsif($action eq 'edit')
     {
-        $form->constraint({ name => 'user', type => 'Required' }); 
+        my $user = $form->get_element({ name => 'user' });
+        $user->attributes->{readonly} = 'readonly';
+        $user->attributes->{disabled} = 'disabled';
     }
     elsif($action eq 'password')
     {
@@ -92,24 +90,32 @@ sub form_generic
     {
         $form = add_multilang_fields($form, $languages, $multilang_conf); 
     }
-    my $category_block = $form->get_element({ name => 'categoryblock'});
-    my $category = $category_block ?
-        $category_block->get_element({ name => 'category'}) :
-        $form->get_element({ name => 'category'});
-    if($category)
-    {
-       $category->options(Strehler::Meta::Category->make_select());
-       my $subcategory = $category_block ?
-            $category_block->get_element({ name => 'subcategory'}) :
-            $form->get_element({ name => 'subcategory'});
-       if($subcategory)
-       {
-           $subcategory->options(Strehler::Meta::Category->make_select($has_sub));
-       }
-    }
     return $form;
 }
 
+sub form_filter
+{
+    my $conf = shift;
+    my $multilang = shift;
+    my $selected_language = shift;
+    my $languages = shift;
+    my @languages_options;
+    for my $l (@{$languages})
+    {
+        push @languages_options, [ $l, $l ];
+    }
+    my @selected_languages = split( ',', $selected_language );
+    my $form = HTML::FormFu->new;
+    $form->load_config_file( $conf );
+    if($multilang)
+    {
+        my $languages_block = $form->get_element({ name => 'language-block'});
+        my $position = $languages_block->get_elements()->[0];
+        $languages_block->insert_after($languages_block->element(
+            { label => "Languages:", type => 'Checkboxgroup', name => 'language', options => \@languages_options, default => $selected_language, attributes => { class => 'languages-filter'}}), $position);
+    }
+    return $form;
+}
 
 sub tags_for_form
 {
@@ -125,12 +131,12 @@ sub tags_for_form
         {
             $params_hashref->{'tags'} = $params_hashref->{'configured-tag'};
         }
-        my $position = $form->get_element({ name => 'categoryblock'}) || $form->get_element({ name => 'subcategory'});
+        my $position = $form->get_elements()->[0]; 
         $form->insert_after($form->element({ type => 'Text', name => 'tags'}), $position);
     }
     elsif($params_hashref->{'tags'})
     { 
-        my $position = $form->get_element({ name => 'categoryblock'}) || $form->get_element({ name => 'subcategory'});
+        my $position = $form->get_elements()->[0]; 
         $form->insert_after($form->element({ type => 'Text', name => 'tags'}), $position);
     }
     return $form;
@@ -217,6 +223,8 @@ Functions that generate forms:
 
 =item form_generic
 
+=item form_filter
+
 =back
 
 tags_for_form is used to manage tags field because it has two different behaviours: as text field and as list of checkboxes.
