@@ -1,5 +1,5 @@
 package Strehler::Meta::Category;
-$Strehler::Meta::Category::VERSION = '1.5.5';
+$Strehler::Meta::Category::VERSION = '1.6.0';
 use strict;
 use Moo;
 use Dancer2;
@@ -34,7 +34,7 @@ sub BUILDARGS {
         my %hash_args =  @args;
         if($hash_args{'parent'})
         {
-            my $main = $class->get_schema()->resultset('Category')->find({ category => $hash_args{'parent'}, parent => undef });
+            my $main = $class->get_schema()->resultset('Category')->find($hash_args{'parent'});
             if($main)
             {
                 $category = $main->categories->find({ category => $hash_args{'category'}});
@@ -67,11 +67,27 @@ sub get_schema
 sub subcategories
 {
     my $self = shift;
+    my $deep = shift;
+    my $already_collected = shift;
     my @subs;
-    return () if ! $self->row;
+    if($already_collected)
+    {
+        @subs = @{$already_collected};
+    }
+
+    return @subs if ! $self->row;
     for($self->row->categories)
     {
-        push @subs, Strehler::Meta::Category->new('row', $_);
+        my $category = Strehler::Meta::Category->new('row', $_);
+        push @subs, $category;
+        if($deep)
+        {
+            @subs = $category->subcategories(1, \@subs);
+        }
+        else
+        {
+            return @subs;
+        }
     }
     return @subs;
 }
@@ -85,19 +101,7 @@ sub get_basic_data
     $data{'name'} = $self->get_attr('category');
     $data{'title'} = $self->get_attr('category');
     $data{'ext_name'} = $self->ext_name;
-    if(! $self->get_attr('parent'))
-    {
-        my @subs = $self->subcategories();
-        if($#subs != -1)
-        {
-            $data{'subcategories'} = [];
-            for(@subs)
-            {
-                my %subdata = $_->get_basic_data();
-                push @{$data{'subcategories'}}, \%subdata;
-            }
-        }
-    }
+    $data{'parent'} = $self->get_attr('parent');
     return %data;
 }
 
@@ -135,14 +139,18 @@ sub max_article_order
 sub delete
 {
     my $self = shift;
+    return -1 if($self->has_elements());
+    return -2 if($self->is_parent());
     $self->row->configured_tags->delete_all();
     $self->row->delete();
+    return 0;
 }
 
 sub get_attr
 {
     my $self = shift;
     my $attr = shift;
+    return undef if(! $self->exists);
     return $self->row->get_column($attr);
 }
 
@@ -158,7 +166,7 @@ sub make_select
     push @base_select, { value => undef, label => "-- select --" }; 
     if($parent && $option  && $option eq 'ancestor')
     {
-        push @category_values_for_select, { value => '*', label => "-- all --" }; 
+        push @category_values_for_select, { value => 'anc:'.$parent, label => "-- all --" }; 
     }
     my $category_count = 0;
     for(@category_values)
@@ -193,6 +201,7 @@ sub get_list
     $args{'order'} ||= 'desc';
     $args{'order_by'} ||= 'id';
     $args{'parent'} ||= undef;
+    $args{'depth'} ||= 0;
     my $search_criteria = undef;
 
     my @to_view;
@@ -201,24 +210,64 @@ sub get_list
     {
         my $cat = Strehler::Meta::Category->new($_->id);
         my %el = $cat->get_basic_data();
+        $el{'depth'} = $args{'depth'};
+        if($args{'depth'} > 5)
+        {
+            $el{'display_name'} = $el{'ext_name'};
+            $el{'display_name'} =~ s/^.*\/(.*\/.*)$/\.\.\.\/$1/;
+        }
+        else
+        {
+            $el{'display_name'} = $el{'name'};
+        }
         push @to_view, \%el;
+        my @subs = $self->get_list({ parent => $_->id, depth => $args{'depth'} + 1});
+        push @to_view, @subs;
     }
-    return  \@to_view;
+    return  @to_view;
 }
 sub explode_name
 {
     my $self = shift;
     my $category_path = shift;
-    my @cats = split '/', $category_path;
-    if(exists $cats[1])
+    return Strehler::Meta::Category->new(-1) if(! $category_path);
+    my $cat_id = $self->name_crawler($category_path, undef);
+    if($cat_id)
     {
-        return Strehler::Meta::Category->new(parent => $cats[0], category => $cats[1]);
+        return Strehler::Meta::Category->new($cat_id);
     }
     else
     {
-        return Strehler::Meta::Category->new(category => $cats[0], parent => undef);
+        return Strehler::Meta::Category->new(-1);
     }
 }
+sub name_crawler
+{
+    my $self = shift;
+    my $name = shift;
+    my $parent = shift;
+
+    my @subnames = split '/', $name;
+    my $cat = shift @subnames;
+    my $cat_obj = Strehler::Meta::Category->new(category => $cat, parent => $parent);
+    $parent ||= '';
+    if($cat_obj->exists())
+    {
+        if(@subnames)
+        {
+            return $self->name_crawler(join('/', @subnames), $cat_obj->get_attr('id'));
+        }
+        else
+        {
+            return $cat_obj->get_attr('id');
+        }
+    }
+    else
+    {
+        return undef;
+    }
+
+}
 
 sub exists
 {
@@ -322,11 +371,13 @@ sub save_form
 sub ext_name
 {
     my $self = shift;
-    return undef if(! $self->row);
+    return undef if(! $self->exists());
     my $category = $self->row->category;
-    if($self->row->parent)
+    my $parent = $self->row->parent;
+    while($parent)
     {
-        $category = $self->row->parent->category . '/' . $category;
+        $category = $parent->category . '/' . $category;
+        $parent = $parent->parent;
     }
     return $category;
 }
@@ -342,6 +393,32 @@ sub check_role
     return ($role eq 'admin');
 }
 
+sub error_message
+{
+    my $self = shift;
+    my $action = shift;
+    my $code = shift;
+    if($code == '0')
+    {
+        return "OK";
+    }
+    else
+    {
+        if($action eq 'delete')
+        {
+            if($code == -1)
+            {
+                return "Category " . $self->get_attr('category') . " is not empty! Deletion is impossible.";    
+            }
+            elsif($code == -2)
+            {
+                return "Category " . $self->get_attr('category') . " has subcategories! Deletion is impossible.";    
+            }
+        }
+        return "An error has occurred";
+    }
+}
+
 
 1;
 
